"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
exports.__esModule = true;
var resolveConfig_1 = __importDefault(require("tailwindcss/resolveConfig"));
var flattenColorPalette_1 = __importDefault(require("tailwindcss/lib/util/flattenColorPalette"));
var properties_1 = require("./properties");
var Tailwind = function (config) {
    // @ts-ignore resolveConfig doesn't like empty config but stubs it anyways
    var resolvedConfig = (0, resolveConfig_1["default"])(config || {});
    var theme = resolvedConfig.theme || {};
    // add negative values to scales
    Object.keys(properties_1.properties).forEach(function (property) {
        var _a = properties_1.properties[property], scale = _a.scale, supportsNegativeValues = _a.supportsNegativeValues;
        if (supportsNegativeValues && theme[scale] && !theme[scale].negativeValuesAdded) {
            Object.keys(theme[scale]).forEach(function (key) {
                theme[scale]['-' + key] = '-' + theme[scale][key];
            });
            theme[scale].negativeValuesAdded = true;
        }
    });
    // TODO: check source code for this because the types are more flexible than object
    var responsiveModifiers = Object.keys(theme.screens || {});
    var pseudoModifiers = resolvedConfig.variantOrder;
    var parse = function (className) {
        if (className === void 0) { className = ''; }
        if (className.startsWith('.'))
            className = className.replace('.', '');
        // format: prefix-value | responsive:prefix-value | pseudo:prefix-value | responsive:pseudo:prefix-value
        var responsiveModifier = null;
        var pseudoModifier = null;
        var propertyName;
        var propertyValue;
        var relatedProperties = {};
        var classNameWithoutModifers = '';
        var numberOfModifiers = className.split(':').length - 1;
        if (numberOfModifiers === 0)
            classNameWithoutModifers = className;
        else if (numberOfModifiers === 1) {
            var unknownModifier = className.split(':')[0];
            classNameWithoutModifers = className.split(':')[1];
            if (responsiveModifiers.includes(unknownModifier))
                responsiveModifier = unknownModifier;
            else if (pseudoModifiers.includes(unknownModifier))
                pseudoModifier = unknownModifier;
            else
                ; // have no idea what this is, TODO: should this ignore or throw an error?
        }
        else if (numberOfModifiers === 2) {
            responsiveModifier = className.split(':')[0];
            pseudoModifier = className.split(':')[1];
            classNameWithoutModifers = className.split(':')[2];
        }
        var isNegative = false;
        if (classNameWithoutModifers.startsWith('-')) {
            isNegative = true;
            classNameWithoutModifers = classNameWithoutModifers.replace('-', '');
        }
        // check named classes first
        if (properties_1.namedClassProperties[classNameWithoutModifers]) {
            var styles = properties_1.namedClassProperties[classNameWithoutModifers];
            if (Object.keys(styles).length > 1) {
                propertyName = 'composite';
                propertyValue = null;
                relatedProperties = styles;
            }
            else {
                propertyName = Object.keys(styles)[0];
                propertyValue = styles[propertyName];
            }
        }
        else {
            var possiblePropertyNames = Object.keys(properties_1.properties).filter(function (name) {
                var property = properties_1.properties[name];
                if (classNameWithoutModifers === property.prefix)
                    return true; // flex-grow-DEFAULT = flex-grow
                if (classNameWithoutModifers.startsWith(property.prefix + '-'))
                    return true;
                return false;
            });
            if (possiblePropertyNames.length === 0) {
                // no clue what this is then
                // TODO: improve error for unhandled properties
                propertyName = 'ERROR';
                propertyValue = 'ERROR';
            }
            else {
                // match value to find property
                var matchingPropertyName = possiblePropertyNames
                    .sort(function (a, b) { return properties_1.properties[b].prefix.length - properties_1.properties[a].prefix.length; })
                    .find(function (name) {
                    var property = properties_1.properties[name];
                    // flatten color scales
                    var scale = property.scale.includes('color') || property.scale.includes('Color')
                        ? (0, flattenColorPalette_1["default"])(theme[property.scale])
                        : theme[property.scale];
                    if (!scale)
                        return false; // couldn't find scale for property, probably unhandled
                    var scaleKey = property.scale === 'colors'
                        ? // remove opacity modifier
                            classNameWithoutModifers.split('/')[0].replace(property.prefix + '-', '')
                        : classNameWithoutModifers.replace(property.prefix + '-', '');
                    if (scale.DEFAULT)
                        scale[property.prefix] = scale.DEFAULT;
                    var possibleValue = scale[scaleKey];
                    // this could be null if it's not the right property
                    return Boolean(possibleValue);
                });
                if (matchingPropertyName) {
                    propertyName = matchingPropertyName;
                    var property = properties_1.properties[matchingPropertyName];
                    var scale = property.scale.includes('color') || property.scale.includes('Color')
                        ? (0, flattenColorPalette_1["default"])(theme[property.scale])
                        : theme[property.scale];
                    var scaleKey = property.scale === 'colors'
                        ? // remove opacity modifier
                            classNameWithoutModifers.split('/')[0].replace(property.prefix + '-', '')
                        : classNameWithoutModifers.replace(property.prefix + '-', '');
                    var possibleValue = scale[scaleKey];
                    // fontSize is special
                    if (propertyName === 'fontSize' && Array.isArray(possibleValue)) {
                        propertyValue = possibleValue[0];
                        relatedProperties = possibleValue[1];
                    }
                    else if (property.scale === 'colors') {
                        var opacity = parseInt(classNameWithoutModifers.split('/')[1]);
                        propertyValue = possibleValue + (opacity ? percentToHex(opacity) : '');
                    }
                    else if (Array.isArray(possibleValue)) {
                        // true for fontFamily and dropShadow
                        propertyValue = possibleValue.join(', ');
                    }
                    else {
                        propertyValue = possibleValue;
                    }
                }
                else {
                    // no clue what this is then
                    propertyName = 'ERROR';
                    propertyValue = 'ERROR';
                }
            }
        }
        return {
            className: className,
            responsiveModifier: responsiveModifier,
            pseudoModifier: pseudoModifier,
            property: propertyName,
            value: isNegative ? '-' + propertyValue : propertyValue,
            relatedProperties: relatedProperties
        };
    };
    var classname = function (_a) {
        var responsiveModifier = _a.responsiveModifier, pseudoModifier = _a.pseudoModifier, propertyName = _a.property, propertyValue = _a.value;
        var className = '';
        var error = {};
        if (responsiveModifier) {
            if (responsiveModifiers.includes(responsiveModifier))
                className = responsiveModifier + ':';
            else
                error['responsiveModifier'] = "Unidentified responsive modifier, expected one of [".concat(responsiveModifiers.join(', '), "], got ").concat(responsiveModifier);
        }
        if (pseudoModifier) {
            if (pseudoModifiers.includes(pseudoModifier))
                className = className + pseudoModifier + ':';
            else
                error['pseudoModifier'] = "Unidentified pseudo modifier, expected one of [".concat(pseudoModifiers.join(', '), "], got ").concat(pseudoModifier);
        }
        var matchingProperty = properties_1.properties[propertyName];
        if (matchingProperty) {
            // flatten color scales
            var scale_1 = matchingProperty.scale.includes('color') || matchingProperty.scale.includes('Color')
                ? (0, flattenColorPalette_1["default"])(theme[matchingProperty.scale])
                : theme[matchingProperty.scale];
            // find value on scale
            if (scale_1) {
                var scaleKey = void 0;
                if (propertyName === 'fontSize') {
                    // format: sm: [ '0.875rem', { lineHeight: '1.25rem' } ],
                    scaleKey = Object.keys(scale_1).find(function (key) { return scale_1[key][0] === propertyValue; });
                }
                else if (matchingProperty.scale === 'colors') {
                    if (!propertyValue.startsWith('#')) {
                        error['value'] = 'Only hex values are supported, example: #fecaca80';
                    }
                    else if (![7, 9].includes(propertyValue.length)) {
                        error['value'] =
                            'Shorthand hex values like #0008 are not supported, please pass the full value like #00000080';
                    }
                    var opacity = null;
                    // example: #fecaca80
                    if (propertyValue.length === 9) {
                        opacity = hexToPercent(propertyValue.slice(-2));
                        propertyValue = propertyValue.slice(0, -2);
                    }
                    // convert to lowercase for comparison
                    propertyValue = propertyValue.toLowerCase();
                    scaleKey = Object.keys(scale_1).find(function (key) {
                        return scale_1[key] === propertyValue;
                    });
                    if (scaleKey && opacity)
                        scaleKey = scaleKey + '/' + opacity;
                }
                else {
                    scaleKey = Object.keys(scale_1).find(function (key) {
                        // true for dropShadow and fontFamily
                        if (Array.isArray(scale_1[key]))
                            return scale_1[key].join(', ') === propertyValue;
                        else
                            return scale_1[key] === propertyValue;
                    });
                }
                // move - for negative value to prefix
                var isNegative = scaleKey === null || scaleKey === void 0 ? void 0 : scaleKey.startsWith('-');
                if (isNegative) {
                    className += '-';
                    scaleKey = scaleKey.replace('-', '');
                }
                className += matchingProperty.prefix;
                if (scaleKey === 'DEFAULT') {
                    /* we don't add default */
                }
                else if (scaleKey)
                    className += '-' + scaleKey;
                else if (!error.value)
                    error['value'] = 'UNIDENTIFIED_VALUE';
            }
            else {
                error['property'] = 'UNIDENTIFIED_PROPERTY';
            }
        }
        else {
            var namedClassPropertyIndex = Object.values(properties_1.namedClassProperties).findIndex(function (styles) {
                if (Object.keys(styles).length > 1)
                    return false;
                var name = Object.keys(styles)[0];
                var value = styles[propertyName];
                return name === propertyName && value === propertyValue;
            });
            if (namedClassPropertyIndex !== -1) {
                className = className + Object.keys(properties_1.namedClassProperties)[namedClassPropertyIndex];
            }
            else if (propertyName === 'color') {
                error['property'] = 'UNIDENTIFIED_PROPERTY, did you mean textColor?';
            }
            else {
                error['property'] = 'UNIDENTIFIED_PROPERTY';
            }
        }
        if (Object.keys(error).length > 0)
            return { error: error };
        else
            return { className: className };
    };
    return { parse: parse, classname: classname, meta: { responsiveModifiers: responsiveModifiers, pseudoModifiers: pseudoModifiers, resolvedConfig: resolvedConfig } };
};
exports["default"] = Tailwind;
var percentToHex = function (percent) {
    var intValue = Math.round((percent / 100) * 255); // map percent to nearest integer (0 - 255)
    var hexValue = intValue.toString(16); // get hexadecimal representation
    return hexValue.padStart(2, '0'); // format with leading 0 and upper case characters
};
var hexToPercent = function (hex) {
    return Math.floor((100 * parseInt(hex, 16)) / 255);
};
